﻿namespace TabsStudioDisambiguator
{
    public class Disambiguator : TabsStudioExt.ITabsStudioAddin
    {
        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte)
        {
            engine = engine_;
            engine.TabCreated += OnTabCreated;
        }

        public void OnDisconnection()
        {
            engine.TabCreated -= OnTabCreated;
        }

        private void OnTabCreated(object sender, TabsStudioExt.TabEventArgs e)
        {
            string title = GetNameWithoutExtension(e.Tab.TitleManager.UntransformedTitle);
            System.Collections.Generic.List<TabsStudioExt.Tab> tabsWithSameTitle =
                new System.Collections.Generic.List<TabsStudioExt.Tab>();
            foreach(TabsStudioExt.Tab tab in e.Tabs.TabList)
            {
                if (GetNameWithoutExtension(tab.TitleManager.UntransformedTitle) == title &&
                    GetParentSolutionFolder(tab.TabExtensions[0].Window) != null)
                {
                    tabsWithSameTitle.Add(tab);
                }
            }
            if (tabsWithSameTitle.Count > 1)
            {
                foreach (TabsStudioExt.Tab tab in tabsWithSameTitle)
                {
                    if (NoDisambiguatorTag(tab))
                    {
                        string parent = GetParentSolutionFolder(tab.TabExtensions[0].Window);
                        tab.TitleManager.Transforms.Add(
                            new TabsStudioExt.TitleTransform("^.*$", parent + "/" + "$_", disambiguatorTag));
                        tab.TitleManager.ApplyTransforms();
                    }
                }
            }
        }

        private bool NoDisambiguatorTag(TabsStudioExt.Tab tab)
        {
            foreach(TabsStudioExt.TitleTransform t in tab.TitleManager.Transforms)
            {
                if (t.Id.Equals(disambiguatorTag))
                    return false;
            }
            return true;
        }

        private string GetParentSolutionFolder(EnvDTE.Window w)
        {
            try
            {
                EnvDTE.ProjectItem pi = w.Document.ProjectItem.Collection.Parent as EnvDTE.ProjectItem;
                if (pi != null)
                    return pi.Name;
                string projectName = w.Document.ProjectItem.ContainingProject.Name;
                if (System.IO.Path.IsPathRooted(projectName))
                    projectName = System.IO.Path.GetFileName(System.IO.Path.GetDirectoryName(projectName));
                return projectName;
            } catch (System.Exception) {
            }
            return null;
        }

        private string GetNameWithoutExtension(string name)
        {
            System.Text.RegularExpressions.Regex regex =
                new System.Text.RegularExpressions.Regex("(?<Name>(.*/)?.+?)(?<Ext>\\..+)");
          	System.Text.RegularExpressions.Match match = regex.Match(name);
            string result = match.Groups["Name"].Value;
            if (result.Length == 0)
                return name;
            return result;
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private static string disambiguatorTag = "[Disambiguator]";
    }
}
